package dev.flyfish.framework.beans.meta.parser.chain;

import java.lang.annotation.Annotation;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

/**
 * 抽象的链式操作
 *
 * @param <A> 注解泛型
 * @param <T> 入参类型
 */
public interface BeanPropertyAnnotationChain<A extends Annotation, T> extends AnnotationChainSupport {

    /**
     * 映射转换
     *
     * @param mapper 映射器
     * @param <R>    泛型
     * @return 结果
     */
    <R> BeanPropertyAnnotationChain<A, R> map(Function<T, R> mapper);

    /**
     * 过滤，不满足的将直接empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    BeanPropertyAnnotationChain<A, T> filter(Predicate<T> filter);

    /**
     * 过滤，不满足的将直接empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    BeanPropertyAnnotationChain<A, T> filter(Supplier<Boolean> filter);

    /**
     * 如果存在，则做
     *
     * @param consumer 消费者
     * @return 结果
     */
    BeanPropertyAnnotationChain<A, T> then(Consumer<T> consumer);

    /**
     * 存在时执行逻辑，不需要结果
     *
     * @param handler 处理器
     * @return 结果
     */
    BeanPropertyAnnotationChain<A, T> exists(Runnable handler);

    /**
     * 不存在做的事情
     *
     * @param fallback 补偿逻辑
     * @return 结果
     */
    BeanPropertyAnnotationChain<A, T> empty(Runnable fallback);

    /**
     * 直接获取结果
     *
     * @return 结果
     */
    A synthesize();

    /**
     * 获取缓存的值
     *
     * @return 结果
     */
    T get();
}
