package dev.flyfish.framework.beans.meta.parser.chain.impl;

import dev.flyfish.framework.beans.meta.parser.BeanPropertyAnnotations;
import dev.flyfish.framework.beans.meta.parser.chain.BeanPropertyAnnotationChain;

import java.lang.annotation.Annotation;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

import static dev.flyfish.framework.beans.meta.parser.BeanPropertyAnnotations.cast;

/**
 * 单例的，空实现
 *
 * @param <A> 注解泛型
 * @param <T> 实际泛型
 */
public class EmptyAnnotationChain<A extends Annotation, T> extends BasicAnnotationChainSupport
        implements BeanPropertyAnnotationChain<A, T> {

    public EmptyAnnotationChain(BeanPropertyAnnotations parent) {
        super(parent);
    }

    /**
     * 映射转换
     *
     * @param mapper 映射器
     * @return 结果
     */
    @Override
    public <R> BeanPropertyAnnotationChain<A, R> map(Function<T, R> mapper) {
        return cast(this);
    }

    /**
     * 过滤，不满足的将直接empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> filter(Predicate<T> filter) {
        return this;
    }

    /**
     * 过滤，不满足的将直接empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> filter(Supplier<Boolean> filter) {
        return this;
    }

    /**
     * 如果存在，则做
     *
     * @param consumer 消费者
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> then(Consumer<T> consumer) {
        markParent();
        return this;
    }

    /**
     * 存在时执行逻辑，不需要结果
     *
     * @param handler 处理器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> exists(Runnable handler) {
        markParent();
        return this;
    }

    /**
     * 不存在做的事情
     *
     * @param fallback 补偿逻辑
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationChain<A, T> empty(Runnable fallback) {
        if (!parent.last()) {
            fallback.run();
        }
        return this;
    }

    /**
     * 当前值是否存在
     *
     * @return 结果
     */
    @Override
    public boolean isPresent() {
        return false;
    }

    /**
     * 直接获取结果
     *
     * @return 结果
     */
    @Override
    public A synthesize() {
        return null;
    }

    /**
     * 获取当前数据
     *
     * @return 结果
     */
    @Override
    public T get() {
        // 从空的占位访问，访问上次的结果
        return parent.value();
    }

    /**
     * 缓存值逻辑
     *
     * @return 父亲
     */
    @Override
    protected BeanPropertyAnnotations cache() {
        return parent;
    }

    /**
     * 需要判定之前的值。当我们进入空链，且调用了then，
     * 代表在没值的时候下次尝试用or时仍可获取值（and总是成立）
     */
    private void markParent() {
        // 当且仅当链上没有值，才尝试置为false，供后续使用
        if (null == parent.value()) {
            parent.last(false);
        }
    }
}
