package dev.flyfish.framework.annotations;

import org.springframework.core.annotation.AliasFor;
import org.springframework.data.mongodb.core.mapping.Field;
import org.springframework.data.relational.core.mapping.Column;

import java.lang.annotation.*;

/**
 * bean的属性注解，提供描述
 *
 * @author wangyu
 */
@Target({ElementType.FIELD, ElementType.TYPE})
@Retention(RetentionPolicy.RUNTIME)
@Documented
@Repeatable(Properties.class)
@Column
@Field
public @interface Property {

    /**
     * 手动指定key，用于覆盖模式
     *
     * @return 结果
     */
    @AliasFor(annotation = Field.class, attribute = "value")
    String key() default "";

    /**
     * 指定数据库列名
     *
     * @return 结果
     */
    @AliasFor(annotation = Column.class, attribute = "value")
    String column() default "";

    /**
     * 显示标题
     *
     * @return 结果
     */
    @AliasFor("title")
    String value() default "";

    /**
     * 显示标题（别名）
     *
     * @return 结果
     */
    @AliasFor("value")
    String title() default "";

    /**
     * 描述
     *
     * @return 结果
     */
    String description() default "";

    /**
     * 被继承的，用于父类，自动拼接名称
     *
     * @return 结果
     */
    boolean inherited() default false;

    /**
     * 是否只读
     *
     * @return 结果
     */
    boolean readonly() default false;

    /**
     * 排序
     *
     * @return 结果
     */
    int order() default 0;

    /**
     * 分组
     *
     * @return 结果
     */
    String group() default "";

    /**
     * 表单注解
     *
     * @return 表单注解，仅用于构造器
     */
    FormItem[] form() default {};

    /**
     * 表单映射
     *
     * @return 映射的注解，仅用于构造器
     */
    MappedTo[] mapping() default {};

    /**
     * 自动生成的值
     */
    Generation[] generated() default {};
}
