package dev.flyfish.framework.mongodb.query;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.mongodb.query.adaptor.MongoCriteriaAdaptor;
import dev.flyfish.framework.mongodb.query.converter.MongoQueryConverter;
import dev.flyfish.framework.query.spi.QueryFactory;
import dev.flyfish.framework.query.spi.adaptor.CriteriaAdaptor;
import dev.flyfish.framework.query.spi.converter.QueryConverter;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.repository.query.MongoEntityInformation;
import org.springframework.data.repository.core.EntityInformation;

/**
 * mongo查询工厂
 *
 * @author wangyu
 */
public class MongoQueryFactory implements QueryFactory<Criteria, Query> {

    private final CriteriaAdaptor<Criteria> adaptor = new MongoCriteriaAdaptor();

    private final QueryConverter<Query> converter = new MongoQueryConverter(adaptor);

    @Override
    public boolean supports(EntityInformation<? extends Domain, String> entity) {
        return entity instanceof MongoEntityInformation;
    }

    @Override
    public QueryConverter<Query> getConverter() {
        return converter;
    }

    @Override
    public CriteriaAdaptor<Criteria> getAdaptor() {
        return adaptor;
    }
}
