package dev.flyfish.framework.r2dbc.metadata.reference.field;

import dev.flyfish.framework.domain.base.Domain;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;

/**
 * 默认访问器实现
 *
 * @author wangyu
 */
class DefaultFieldAccessor implements FieldAccessor {

    private final PropertyDescriptor descriptor;

    DefaultFieldAccessor(PropertyDescriptor descriptor) {
        this.descriptor = descriptor;
    }

    /**
     * 设置字段值
     *
     * @param obj   具体对象
     * @param value 值
     * @return 设置后的对象
     */
    @Override
    public <T extends Domain> T set(T obj, Object value) {
        if (null != descriptor.getWriteMethod()) {
            invokeSetter(descriptor.getWriteMethod(), obj, value);
        }
        return obj;
    }

    /**
     * 取得字段值
     *
     * @param obj 具体对象
     * @return 具体字段值
     */
    @Override
    public <T extends Domain, R> R get(T obj) {
        if (null != descriptor.getReadMethod()) {
            return invokeGetter(descriptor.getReadMethod(), obj);
        }
        return null;
    }

    /**
     * setter是否可用
     *
     * @return 结果
     */
    @Override
    public boolean hasSetter() {
        return null != descriptor.getWriteMethod();
    }

    /**
     * getter是否可用
     *
     * @return 结果
     */
    @Override
    public boolean hasGetter() {
        return null != descriptor.getReadMethod();
    }

    private void invokeSetter(Method method, Object obj, Object value) {
        try {
            method.invoke(obj, value);
        } catch (Exception ignored) {

        }
    }

    @SuppressWarnings("unchecked")
    private <T> T invokeGetter(Method method, Object obj) {
        try {
            return (T) method.invoke(obj);
        } catch (Exception ignored) {
            return null;
        }
    }
}
