package dev.flyfish.framework.r2dbc.query.adaptor;

import dev.flyfish.framework.utils.JacksonUtil;
import lombok.Getter;
import org.springframework.data.relational.core.query.Criteria;
import org.springframework.data.relational.core.query.CriteriaDefinition;
import org.springframework.data.relational.core.sql.SqlIdentifier;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * 函数查询
 *
 * @author wangyu
 */
@Getter
public class FunctionCriteria implements CriteriaDefinition {

    private final CriteriaDefinition previous;

    private final Combinator combinator;

    private final SqlIdentifier column;

    private final String functionName;

    private final List<Object> values;

    private FunctionCriteria(SqlIdentifier column, String functionName, Object... values) {
        this(null, null, column, functionName, values);
    }

    private FunctionCriteria(@Nullable CriteriaDefinition previous, @Nullable Combinator combinator,
                             SqlIdentifier column, String functionName, Object... values) {
        this.previous = previous;
        this.combinator = combinator;
        this.column = column;
        this.functionName = functionName;
        this.values = Arrays.asList(values);
    }

    public static FunctionStep where(String column) {
        return new DefaultFunctionStep(column);
    }

    @Override
    public boolean isGroup() {
        return false;
    }

    @Override
    @NonNull
    public List<CriteriaDefinition> getGroup() {
        return Collections.emptyList();
    }

    @Override
    public Comparator getComparator() {
        return Comparator.INITIAL;
    }

    @Override
    public Object getValue() {
        return values;
    }

    @Override
    public boolean isIgnoreCase() {
        return false;
    }

    @Override
    public boolean hasPrevious() {
        return null != previous;
    }

    @Override
    public boolean isEmpty() {
        return false;
    }

    public Criteria and() {
        return Criteria.empty().and(this);
    }

    public Criteria or() {
        return Criteria.empty().or(this);
    }

    public FunctionStep and(String column) {
        return new DefaultFunctionStep(column, this, Combinator.AND);
    }

    public FunctionStep or(String column) {
        return new DefaultFunctionStep(column, this, Combinator.OR);
    }

    /**
     * 主要操作步骤，支持若干函数
     */
    public interface FunctionStep {

        /**
         * json包含，mysql特有
         *
         * @param value 值
         * @return 结果
         */
        FunctionCriteria jsonContains(Object value);
    }

    /**
     * 默认的查询链实现
     */
    private static class DefaultFunctionStep implements FunctionStep {

        private final CriteriaDefinition previous;

        private final SqlIdentifier column;

        private final Combinator combinator;

        private DefaultFunctionStep(String column) {
            this(column, Criteria.empty(), Combinator.INITIAL);
        }

        private DefaultFunctionStep(String column, CriteriaDefinition previous, Combinator combinator) {
            this.column = SqlIdentifier.quoted(column);
            this.previous = previous;
            this.combinator = combinator;
        }

        @Override
        public FunctionCriteria jsonContains(Object value) {
            String represent = JacksonUtil.toJson(value).orElse(String.valueOf(value));
            return create(column, "JSON_CONTAINS", represent);
        }

        private FunctionCriteria create(SqlIdentifier column, String functionName, Object... values) {
            return new FunctionCriteria(previous, combinator, column, functionName, values);
        }
    }
}
