package dev.flyfish.framework.beans.repository;

import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanNameGenerator;
import org.springframework.core.env.Environment;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.type.AnnotationMetadata;

/**
 * 仓库注册器
 *
 * @author wangyu
 */
public interface RepositoryRegistrar {

    /**
     * 初始化注册器，不初始化无法使用
     *
     * @param metadata       元数据
     * @param resourceLoader 资源加载器
     * @param environment    环境变量
     * @param registry       bean注册器
     * @param generator      生成器
     * @return 结果
     */
    RepositoryRegistrar init(AnnotationMetadata metadata, ResourceLoader resourceLoader, Environment environment,
                             BeanDefinitionRegistry registry, BeanNameGenerator generator);

    /**
     * 根据class直接注册bean
     *
     * @param clazz 编译后的class
     */
    void register(Class<?> clazz);

    /**
     * 创建新的注册器
     *
     * @return 注册器
     */
    static RepositoryRegistrar newInstance(AnnotationMetadata metadata) {
        return new RepositoryRegistrarComposite(metadata);
    }
}
